@@ -75,7 +75,7 @@ class user_loader
 		// Do not load users we already have in $this->users
 		$user_ids = array_diff($user_ids, array_keys($this->users));
 
-		if (sizeof($user_ids))
+		if (count($user_ids))
 		{
 			$sql = 'SELECT *
 				FROM ' . $this->users_table . '
@@ -175,7 +175,7 @@ class user_loader
 	/**
 	* Get avatar
 	*
-	* @param int $user_id User ID of the user you want to retreive the avatar for
+	* @param int $user_id User ID of the user you want to retrieve the avatar for
 	* @param bool $query Should we query the database if this user has not yet been loaded?
 	* 						Typically this should be left as false and you should make sure
 	* 						you load users ahead of time with load_users()
@@ -189,12 +189,14 @@ class user_loader
 			return '';
 		}
 
-		if (!function_exists('get_user_avatar'))
-		{
-			include($this->phpbb_root_path . 'includes/functions_display.' . $this->php_ext);
-		}
+		$row = array(
+			'avatar'		=> $user['user_avatar'],
+			'avatar_type'	=> $user['user_avatar_type'],
+			'avatar_width'	=> $user['user_avatar_width'],
+			'avatar_height'	=> $user['user_avatar_height'],
+		);
 
-		return get_user_avatar($user['user_avatar'], $user['user_avatar_type'], $user['user_avatar_width'], $user['user_avatar_height'], 'USER_AVATAR', false, $lazy);
+		return phpbb_get_avatar($row, 'USER_AVATAR', false, $lazy);
 	}
 
 	/**
